@@ -9,29 +9,27 @@
 // or else upgrading old project file will not work
 // reliably.
 // TODO QGIS 4: remove in favor of _QGIS_VERSION
-#define VERSION "3.34.1-Prizren"
-#define _QGIS_VERSION "3.34.1-Prizren"
+#define VERSION "3.40.1-Bratislava"
+#define _QGIS_VERSION "3.40.1-Bratislava"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 3341
+//#define VERSION_INT 3401
 // TODO QGIS 4: Remove in favor of _QGIS_VERSION_INT
-#define VERSION_INT 33401
-#define _QGIS_VERSION_INT 33401
-#define ABISYM(x) x ## 33401
+#define VERSION_INT 34001
+#define _QGIS_VERSION_INT 34001
+#define ABISYM(x) x ## 34001
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "Prizren"
+#define RELEASE_NAME "Bratislava"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
 #define QGIS_LIBEXEC_SUBDIR "lib/qgis"
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
-#define CMAKE_INSTALL_PREFIX "/usr"
-#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.34.1"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
@@ -44,15 +42,13 @@
 #define QGIS_MACAPP_BUNDLE 
 #endif
 
-#define QT_PLUGINS_DIR "/usr/lib/qt/plugins"
-
-#define PYTHON_VERSION "3.11.6"
+#define PYTHON_VERSION "3.12.7"
 #define PYTHON_VERSION_MAJOR "3"
-#define PYTHON_VERSION_MINOR "11"
-#define PYTHON_VERSION_PATCH "6"
+#define PYTHON_VERSION_MINOR "12"
+#define PYTHON_VERSION_PATCH "7"
 
 #define PROJ_VERSION_MAJOR 9
-#define PROJ_VERSION_MINOR 3
+#define PROJ_VERSION_MINOR 5
 #define PROJ_VERSION_PATCH 0
 
 /* #undef USING_NMAKE */
@@ -98,6 +94,8 @@
 
 /* #undef HAVE_QTGAMEPAD */
 
+#define HAVE_QTPRINTER
+
 /* #undef HAVE_STATIC_PROVIDERS */
 
 #define HAVE_EPT
@@ -105,12 +103,12 @@
 #define HAVE_COPC
 
 #define HAVE_PDAL_QGIS
-#define PDAL_VERSION "2.6.1"
+#define PDAL_VERSION "2.8.0"
 #define PDAL_VERSION_MAJOR_INT 2
-#define PDAL_VERSION_MINOR_INT 6
+#define PDAL_VERSION_MINOR_INT 8
 #define PDAL_VERSION_MAJOR "2"
-#define PDAL_VERSION_MINOR "6"
-#define PDAL_VERSION_MICRO "1"
+#define PDAL_VERSION_MINOR "8"
+#define PDAL_VERSION_MICRO "0"
 
 #define HAVE_DRACO
 
@@ -120,7 +118,11 @@
 #define HAS_KDE_QT5_SMALL_CAPS_FIX
 #define HAS_KDE_QT5_FONT_STRETCH_FIX
 
+/* #undef HAVE_PDF4QT */
+
 #define HAVE_CRASH_HANDLER
 
+/* #undef HAVE_WEBENGINE */
+
 #endif
 
