@@ -20,15 +20,13 @@
 __author__ = 'Nathan Woodrow'
 __date__ = 'January 2015'
 __copyright__ = '(C) 2015, Nathan Woodrow'
-# This will get replaced with a git SHA1 when you do a git archive
-__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
 
 import os
 import sys
 import glob
 import traceback
 
-from qgis.PyQt.QtCore import QCoreApplication
+from qgis.PyQt.QtCore import QCoreApplication, qDebug
 from qgis.core import Qgis, QgsApplication, QgsMessageLog
 
 
@@ -36,7 +34,7 @@ def load_user_expressions(path):
     """
     Load all user expressions from the given paths
     """
-    #Loop all py files and import them
+    # Loop all py files and import them
     modules = glob.glob(path + "/*.py")
     names = [os.path.basename(f)[:-3] for f in modules]
     for name in names:
@@ -50,7 +48,42 @@ def load_user_expressions(path):
             error = traceback.format_exc()
             msgtitle = QCoreApplication.translate("UserExpressions", "User expressions")
             msg = QCoreApplication.translate("UserExpressions", "The user expression {0} is not valid").format(name)
-            QgsMessageLog.logMessage(msg + "\n" + error, msgtitle, Qgis.Warning)
+            QgsMessageLog.logMessage(msg + "\n" + error, msgtitle, Qgis.MessageLevel.Warning)
+
+
+def reload_user_expressions(path):
+    """
+    Reload all user expressions from the given path
+    """
+    # First unload expression modules, looping all
+    # py files and remove them from sys.modules
+    modules = glob.glob(path + "/*.py")
+    names = [os.path.basename(f)[:-3] for f in modules]
+    for name in names:
+        if name == "__init__":
+            continue
+
+        mod = "expressions.{0}".format(name)
+        if mod not in sys.modules:
+            continue
+
+        # try removing path
+        if hasattr(sys.modules[mod], '__path__'):
+            for path in sys.modules[mod].__path__:
+                try:
+                    sys.path.remove(path)
+                except ValueError:
+                    # Discard if path is not there
+                    pass
+
+        # try to remove the module from python
+        try:
+            del sys.modules[mod]
+        except:
+            qDebug("Error when removing module:\n%s" % traceback.format_exc())
+
+    # Finally, load again the users expressions from the given path
+    load_user_expressions(path)
 
 
 userpythonhome = os.path.join(QgsApplication.qgisSettingsDirPath(), "python")
@@ -68,8 +101,8 @@ if not os.path.exists(initfile):
 template = """from qgis.core import *
 from qgis.gui import *
 
-@qgsfunction(args='auto', group='Custom')
-def my_sum(value1, value2, feature, parent):
+@qgsfunction(group='Custom', referenced_columns=[])
+def my_sum(value1, value2):
     \"\"\"
     Calculates the sum of the two parameters value1 and value2.
     <h2>Example usage:</h2>
@@ -83,13 +116,13 @@ def my_sum(value1, value2, feature, parent):
 
 default_expression_template = template
 
-
 try:
     import expressions
 
     expressions.load = load_user_expressions
     expressions.load(expressionspath)
     expressions.template = template
+    expressions.reload = reload_user_expressions
 except ImportError:
     # We get a import error and crash for some reason even if we make the expressions package
     # TODO Fix the crash on first load with no expressions folder
