@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     RandomPointsAlongLines.py
@@ -21,14 +19,11 @@ __author__ = 'Alexander Bruy'
 __date__ = 'April 2014'
 __copyright__ = '(C) 2014, Alexander Bruy'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
-
 import random
 
-from qgis.PyQt.QtCore import QVariant
-from qgis.core import (QgsField,
+from qgis.PyQt.QtCore import QMetaType
+from qgis.core import (Qgis,
+                       QgsField,
                        QgsFeatureSink,
                        QgsFeature,
                        QgsFields,
@@ -52,7 +47,6 @@ from processing.tools import vector
 
 
 class RandomPointsAlongLines(QgisAlgorithm):
-
     INPUT = 'INPUT'
     POINTS_NUMBER = 'POINTS_NUMBER'
     MIN_DISTANCE = 'MIN_DISTANCE'
@@ -70,17 +64,17 @@ class RandomPointsAlongLines(QgisAlgorithm):
     def initAlgorithm(self, config=None):
         self.addParameter(QgsProcessingParameterFeatureSource(self.INPUT,
                                                               self.tr('Input layer'),
-                                                              [QgsProcessing.TypeVectorLine]))
+                                                              [QgsProcessing.SourceType.TypeVectorLine]))
         self.addParameter(QgsProcessingParameterNumber(self.POINTS_NUMBER,
                                                        self.tr('Number of points'),
-                                                       QgsProcessingParameterNumber.Integer,
+                                                       QgsProcessingParameterNumber.Type.Integer,
                                                        1, False, 1, 1000000000))
         self.addParameter(QgsProcessingParameterDistance(self.MIN_DISTANCE,
                                                          self.tr('Minimum distance between points'),
                                                          0, self.INPUT, False, 0, 1000000000))
         self.addParameter(QgsProcessingParameterFeatureSink(self.OUTPUT,
                                                             self.tr('Random points'),
-                                                            type=QgsProcessing.TypeVectorPoint))
+                                                            type=QgsProcessing.SourceType.TypeVectorPoint))
 
     def name(self):
         return 'randompointsalongline'
@@ -88,6 +82,9 @@ class RandomPointsAlongLines(QgisAlgorithm):
     def displayName(self):
         return self.tr('Random points along line')
 
+    def documentationFlags(self):
+        return Qgis.ProcessingAlgorithmDocumentationFlag.RegeneratesPrimaryKey
+
     def processAlgorithm(self, parameters, context, feedback):
         source = self.parameterAsSource(parameters, self.INPUT, context)
         if source is None:
@@ -97,38 +94,47 @@ class RandomPointsAlongLines(QgisAlgorithm):
         minDistance = self.parameterAsDouble(parameters, self.MIN_DISTANCE, context)
 
         fields = QgsFields()
-        fields.append(QgsField('id', QVariant.Int, '', 10, 0))
+        fields.append(QgsField('id', QMetaType.Type.Int, '', 10, 0))
 
         (sink, dest_id) = self.parameterAsSink(parameters, self.OUTPUT, context,
-                                               fields, QgsWkbTypes.Point, source.sourceCrs())
+                                               fields, QgsWkbTypes.Type.Point, source.sourceCrs(), QgsFeatureSink.SinkFlag.RegeneratePrimaryKey)
         if sink is None:
             raise QgsProcessingException(self.invalidSinkError(parameters, self.OUTPUT))
 
         nPoints = 0
         nIterations = 0
         maxIterations = pointCount * 200
-        featureCount = source.featureCount()
         total = 100.0 / pointCount if pointCount else 1
 
         index = QgsSpatialIndex()
-        points = dict()
+        points = {}
 
         da = QgsDistanceArea()
         da.setSourceCrs(source.sourceCrs(), context.transformContext())
-        da.setEllipsoid(context.project().ellipsoid())
+        da.setEllipsoid(context.ellipsoid())
 
         request = QgsFeatureRequest()
 
         random.seed()
 
-        while nIterations < maxIterations and nPoints < pointCount:
+        ids = source.allFeatureIds()
+
+        while ids and nIterations < maxIterations and nPoints < pointCount:
             if feedback.isCanceled():
                 break
 
             # pick random feature
-            fid = random.randint(0, featureCount - 1)
-            f = next(source.getFeatures(request.setFilterFid(fid).setSubsetOfAttributes([])))
+            fid = random.choice(ids)
+            try:
+                f = next(source.getFeatures(request.setFilterFid(fid).setSubsetOfAttributes([])))
+            except StopIteration:
+                ids.remove(fid)
+                continue
+
             fGeom = f.geometry()
+            if fGeom.isEmpty():
+                ids.remove(fid)
+                continue
 
             if fGeom.isMultipart():
                 lines = fGeom.asMultiPolyline()
@@ -139,34 +145,40 @@ class RandomPointsAlongLines(QgisAlgorithm):
                 vertices = fGeom.asPolyline()
 
             # pick random segment
-            if len(vertices) == 2:
+            nVertices = len(vertices)
+            if nVertices < 2:
+                nIterations += 1
+                continue
+            if nVertices == 2:
                 vid = 0
             else:
-                vid = random.randint(0, len(vertices) - 2)
+                vid = random.randint(0, nVertices - 2)
             startPoint = vertices[vid]
             endPoint = vertices[vid + 1]
             length = da.measureLine(startPoint, endPoint)
+            if length == 0:
+                nIterations += 1
+                continue
             dist = length * random.random()
 
-            if dist > minDistance:
-                d = dist / (length - dist)
-                rx = (startPoint.x() + d * endPoint.x()) / (1 + d)
-                ry = (startPoint.y() + d * endPoint.y()) / (1 + d)
+            d = dist / (length - dist)
+            rx = (startPoint.x() + d * endPoint.x()) / (1 + d)
+            ry = (startPoint.y() + d * endPoint.y()) / (1 + d)
 
-                # generate random point
-                p = QgsPointXY(rx, ry)
-                geom = QgsGeometry.fromPointXY(p)
-                if vector.checkMinDistance(p, index, minDistance, points):
-                    f = QgsFeature(nPoints)
-                    f.initAttributes(1)
-                    f.setFields(fields)
-                    f.setAttribute('id', nPoints)
-                    f.setGeometry(geom)
-                    sink.addFeature(f, QgsFeatureSink.FastInsert)
-                    index.addFeature(f)
-                    points[nPoints] = p
-                    nPoints += 1
-                    feedback.setProgress(int(nPoints * total))
+            # generate random point
+            p = QgsPointXY(rx, ry)
+            geom = QgsGeometry.fromPointXY(p)
+            if vector.checkMinDistance(p, index, minDistance, points):
+                f = QgsFeature(nPoints)
+                f.initAttributes(1)
+                f.setFields(fields)
+                f.setAttribute('id', nPoints)
+                f.setGeometry(geom)
+                sink.addFeature(f, QgsFeatureSink.Flag.FastInsert)
+                index.addFeature(f)
+                points[nPoints] = p
+                nPoints += 1
+                feedback.setProgress(int(nPoints * total))
             nIterations += 1
 
         if nPoints < pointCount:
