@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 """
 ***************************************************************************
     DeleteScriptAction.py
@@ -21,10 +19,6 @@ __author__ = 'Victor Olaya'
 __date__ = 'August 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
-
 import os
 
 from qgis.PyQt.QtCore import QCoreApplication
@@ -50,9 +44,9 @@ class DeleteScriptAction(ContextAction):
         reply = QMessageBox.question(None,
                                      self.tr("Delete Script"),
                                      self.tr("Are you sure you want to delete this script?"),
-                                     QMessageBox.Yes | QMessageBox.No,
-                                     QMessageBox.No)
-        if reply == QMessageBox.Yes:
+                                     QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
+                                     QMessageBox.StandardButton.No)
+        if reply == QMessageBox.StandardButton.Yes:
             filePath = ScriptUtils.findAlgorithmSource(self.itemData.name())
             if filePath is not None:
                 os.remove(filePath)
